
/*
     ______     __                     ___     __
    /\  _  \   /\ \                   /\_ \   /\_\
    \ \ \_\ \  \ \ \      __     ___  \//\ \  \/_/_
     \ \  __ \ `/__ \   /'__`\  / __`\  \ \ \   /\ \
      \ \ \/\ \/\\_\ \_/\ \_\ \/\ \_\ \_ \_\ \_ \_\ \_
       \ \_\ \_\ \_____\ \___\ \ \______\/\____\/\____\
        \/_/\/_/\/_____/\/___L\ \/______/\/____/\/____/
                          /\____/
                          \_/__/




                  An advanced game library

                 (c) Stephane APIOU 2003


    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



*/

//! this file describes the endian definition

/*!\file agl_endian.h
 * \ingroup tools
 * \author Stephane APIOU
 *
 */


#ifndef _agl_endian_h
#define _agl_endian_h

/* Set up for C function definitions, even when using C++ */
#ifdef __cplusplus
extern "C"
{
#endif

//! is not NUL if the computer is a big endian design
#define AGL_BIGENDIAN 0

#if AGL_BIGENDIAN

#define agl_swaple16(v) ((((v) & 0XFF00) >> 8) | (((v) & 0XFF) << 8))
#define agl_swaple32(v) (((v) >> 24) | (((v) & 0XFF00) << 8) | (((v) & 0XFF0000) >> 8) | ((v) << 24))
#define agl_swapbe16(v) (v)
#define agl_swapbe32(v) (v)

#else

//! swap from little endian format to internal format an unsigned short value

/*! swap from little endian format to internal format an unsigned short value
 * \param v the value in little endian format
 *
 * \return an unsigned short value
 */

#define agl_swaple16(v) (v)

//! swap from little endian format to internal format an unsigned long value

/*! swap from little endian format to internal format an unsigned long value
 * \param v the value in little endian format
 *
 * \return an unsigned long value
 */

#define agl_swaple32(v) (v)

//! swap from big endian format to internal format an unsigned short value

/*! swap from big endian format to internal format an unsigned short value
 * \param v the value in big endian format
 *
 * \return an unsigned short value
 */

#define agl_swapbe16(v) ((((v) & 0XFF00) >> 8) | (((v) & 0XFF) << 8))

//! swap from big endian format to internal format an unsigned long value

/*! swap from big endian format to internal format an unsigned long value
 * \param v the value in big endian format
 *
 * \return an unsigned long value
 */

#define agl_swapbe32(v) (((v) >> 24) | (((v) & 0XFF00) << 8) | (((v) & 0XFF0000) >> 8) | ((v) << 24))

#endif

//! swap from little endian format to internal format an unsigned short pointer

/*! swap from little endian format to internal format an unsigned short pointer
 * \param ptr the pointer to the little endian format
 *
 * \return an unsigned short value
 */

#define agl_readle16(ptr) ((unsigned short)((((unsigned char *)(ptr))[0]) << 0) | ((((unsigned char *)(ptr))[1]) << 8))

//! swap from little endian format to internal format an unsigned long pointer

/*! swap from little endian format to internal format an unsigned long pointer
 * \param ptr the pointer to the little endian format
 *
 * \return an unsigned long value
 */

#define agl_readle32(ptr) ((unsigned long)((((unsigned char *)(ptr))[0]) << 0) | ((((unsigned char *)(ptr))[1]) << 8) | ((((unsigned char *)(ptr))[2]) << 16) | ((((unsigned char *)(ptr))[3]) << 24))

//! swap from big endian format to internal format an unsigned short pointer

/*! swap from big endian format to internal format an unsigned short pointer
 * \param ptr the pointer to the big endian format
 *
 * \return an unsigned short value
 */

#define agl_readbe16(ptr) ((unsigned short)((((unsigned char *)(ptr))[0]) << 8) | ((((unsigned char *)(ptr))[1]) << 0))

//! swap from big endian format to internal format an unsigned long pointer

/*! swap from big endian format to internal format an unsigned long pointer
 * \param ptr the pointer to the big endian format
 *
 * \return an unsigned long value
 */

#define agl_readbe32(ptr) ((unsigned long)((((unsigned char *)(ptr))[0]) << 24) | ((((unsigned char *)(ptr))[1]) << 16) | ((((unsigned char *)(ptr))[2]) << 8) | ((((unsigned char *)(ptr))[3]) << 0))

/* Ends C function definitions when using C++ */
#ifdef __cplusplus
};
#endif

#endif /* _agl_endian_h */
