
/*
     ______     __                     ___     __
    /\  _  \   /\ \                   /\_ \   /\_\
    \ \ \_\ \  \ \ \      __     ___  \//\ \  \/_/_
     \ \  __ \ `/__ \   /'__`\  / __`\  \ \ \   /\ \
      \ \ \/\ \/\\_\ \_/\ \_\ \/\ \_\ \_ \_\ \_ \_\ \_
       \ \_\ \_\ \_____\ \___\ \ \______\/\____\/\____\
        \/_/\/_/\/_____/\/___L\ \/______/\/____/\/____/
                          /\____/
                          \_/__/




                  An advanced game library

                 (c) Stephane APIOU 2003


    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



*/

#ifndef _agl_object_h
#define _agl_object_h

struct _agl_module;
struct _agl_objectlist;

#define AGL_OBJ_VALID		0X00000001	//!< the object is valid: sub objects exists
#define AGL_OBJ_LOADED		0X00000002	//!< the object is loaded in memory
#define AGL_OBJ_LOCKED		0X00000004	//!< the object is locked and can't be unloaded by system
#define AGL_OBJ_FROZEN		0X00000008	//!< the object is frozen and can't be modified by system

#define AGL_OBJTO_UNFREEZE	0X08000000	//!< the object must be unfrozen if possible
#define AGL_OBJTO_UNLOCK	0X10000000	//!< the object must be unlocked when possible
#define AGL_OBJTO_UNLOAD	0X20000000	//!< the object must be unloaded when possible
#define AGL_OBJTO_UNVALID	0X40000000	//!< the object must be unloaded when possible
#define AGL_OBJTO_DELETE	0X80000000	//!< the object must be deleted if possible

typedef struct _agl_caller
{
	char *name;
	void *priv;
} agl_caller;

#define AGL_MODER_SZ	5

#ifndef _AGL_PRIVATE

typedef struct _agl_module
{
	char *name;		// class name
	char *detailed_name;	// detailed name
	agl_caller caller[AGL_MODER_SZ];	// caller module name
	void *phandle;
	void *priv[11];		// private elements used by class
} agl_module;


typedef struct _agl_object
{
	char *name;
	int flags;
	void *priv;
	agl_module *pmod;
	struct _agl_objectlist *usedby;
	int dum1[2];
	void *dum[4];
} agl_object;


#endif

#define AGL_STYPENONE 0
#define AGL_STYPESTR 1
#define AGL_STYPEDBL 2
#define AGL_STYPEINT 4
#define AGL_STYPELIST 8
#define AGL_STYPEUSER 0X10
#define AGL_STYPEOBJREF 0X20
#define AGL_STYPEALL 0XFFF

#define AGL_STYPESTATIC 0X1000
#define AGL_STYPEALLOC 0X2000

typedef struct _agl_objectlist
{
	agl_object *pobj;
	struct _agl_objectlist *pnext;
} agl_objectlist;

extern agl_object *agl_objcreate(char *name, agl_module * pmod);

extern int agl_objdelete(agl_object * pobj);

//! base type objects

union agl_elemu
{
	char *str;
	double dbl;
	long num;
	struct _agl_elemtype *list;
	agl_object *user;
	struct
	{
		char *name;
		char *mod;
	} ref;

};

typedef struct _agl_elemtype
{
	union agl_elemu vu;
	union agl_elemu iu;
	unsigned short vtype;
	unsigned short itype;
	struct _agl_elemtype *pnext;
} agl_elemtype;

struct agl_eleme
{
	char *str;
	double dbl;
	long num;
	struct _agl_elemtype *list;
	agl_object *user;
	struct
	{
		char *name;
		char *mod;
	} ref;

};

typedef struct _agl_elemexp
{
	struct agl_eleme ve;
	struct agl_eleme ie;
	unsigned short vtype;
	unsigned short itype;
} agl_elemexpand;

#define DEFN_DECLINTVAL(name, val)         { name, { { NULL, 0, val , NULL, NULL, {NULL, NULL} },{ NULL, 0, 0, NULL, NULL, {NULL, NULL } } , AGL_STYPEINT, AGL_STYPENONE}, 0 }
#define DEFN_DECLDBLVAL(name, val)         { name, { { NULL, val, 0 , NULL, NULL, {NULL, NULL} },{ NULL, 0, 0, NULL, NULL, {NULL, NULL } } , AGL_STYPEDBL, AGL_STYPENONE}, 0 }
#define DEFN_DECLSTRVAL(name, val)         { name, { { val, 0, 0 , NULL, NULL, {NULL, NULL} },{ NULL, 0, 0, NULL, NULL, {NULL, NULL } } , AGL_STYPESTR, AGL_STYPENONE}, 0 }
#define DEFN_DECLEND         { NULL, { { NULL, 0, 0 , NULL, NULL, {NULL, NULL} },{ NULL, 0, 0, NULL, NULL, {NULL, NULL } } , AGL_STYPENONE, AGL_STYPENONE}, 0 }

typedef struct _agl_elemdefn agl_elempriv;

extern int agl_getelem(agl_handle handle, agl_elemtype * psrc, agl_elemtype * pdst);
extern int agl_typeidx(agl_object * pobj, int nb, ...);

#endif
