
/*
     ______     __                     ___     __
    /\  _  \   /\ \                   /\_ \   /\_\
    \ \ \_\ \  \ \ \      __     ___  \//\ \  \/_/_
     \ \  __ \ `/__ \   /'__`\  / __`\  \ \ \   /\ \
      \ \ \/\ \/\\_\ \_/\ \_\ \/\ \_\ \_ \_\ \_ \_\ \_
       \ \_\ \_\ \_____\ \___\ \ \______\/\____\/\____\
        \/_/\/_/\/_____/\/___L\ \/______/\/____/\/____/
                          /\____/
                          \_/__/




                  An advanced game library

                 (c) Stephane APIOU 2003


    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



*/

//! definition of all adgali features associated with video functions

/*! \file
 * \ingroup video
 * this file defines all adgali features associated vith the video functions
 * and must be include when you use adgali
 * agl.h includes this file
 */


#ifndef _adgali__video_h
#define _adgali_video_h


//! \name pixel mask for the long representation. pixels stored in memory in RGBA order
//@{

#define AGL_RED		agl_swapbe32(0XFF000000)	//!< red color mask
#define AGL_GREEN	agl_swapbe32(0X00FF0000)	//!< green color mask
#define AGL_BLUE	agl_swapbe32(0X0000FF00)	//!< blue color mask
#define AGL_ALPHA	agl_swapbe32(0X000000FF)	//!< alpha mask

//@}

//! \name macros to point onto each channel into a pixel pointer
//@{
#define AGL_REDVAL(p)	(((unsigned char *)(p))[0])	//!< point onto red value
#define AGL_GREENVAL(p)	(((unsigned char *)(p))[1])	//!< point onto green value
#define AGL_BLUEVAL(p)	(((unsigned char *)(p))[2])	//!< point onto blue value
#define AGL_ALPHAVAL(p)	(((unsigned char *)(p))[3])	//!< point onto alpha value

//@}
//! \name special alpha values
//@{

#define AGL_OPAQUE	0XFF	//!< set alpha opaque
#define AGL_TRANSPARENT	0X00	//!< set alpha transparent


//@}
//
#define AGL_VALRGBA2LONG(red,green, blue, alpha) AGL_RGBA2LONG(((red)<<24)|((green)<<16)|((blue)<<8)|(alpha))	//!< tree colors and alpha to a raw value

#define AGL_RGBA2LONG(value) agl_swapbe32(value)	//!< transcode a long color to a long value

//! set red color

/*! this function sets the red color in a pixel value
 * \param value the color value is between 0 and 255 (0 is black)
 * \return the pixel value
 */

#define AGL_SETRED(value)	(agl_swapbe32(value<<24) & AGL_RED)

//! set green color

/*! this function sets the green color in a pixel value
 * \param value the color value is between 0 and 255 (0 is black)
 * \return the pixel value
 */

#define AGL_SETGREEN(value)	(agl_swapbe32(value << 16) & AGL_GREEN)

//! set blue color

/*! this function sets the blue color in a pixel value
 * \param value the color value is between 0 and 255 (0 is black)
 * \return the pixel value
 */

#define AGL_SETBLUE(value)	(agl_swapbe32(value << 8) & AGL_BLUE)

//! set alpha channel

/*! this function sets the alpha channel in a pixel value
 * \param value the alpha value or this pixel (255 is opaque, 0 is transparent)
 * \return the pixel value
 */

#define AGL_SETALPHA(value)	(agl_swapbe32(value) & AGL_ALPHA)	//!< set alpha

//! \name video mode options
//@{

#define AGL_FULLSCREEN	1	//!< full screen mode

//@}

//! \name specify the type of channel separation or combination to use
//@{

#define AGL_RGB	0		//!< specify a RGB (Red, Green, Blue) combination or separation
#define AGL_CMY	1		//!< specify a CMY (Cyan, Magenta, Yellow) combination or separation
#define AGL_HSV 2		//!< specify an HSV (Hue, Saturation, Value) combination or separation
#define AGL_HLS 3		//!< specify an HLS (Hue, Lightness, Saturation) combination or separation
#define AGL_AVG 4		//!< specify an average ( average of RGB channel ie luminance)

//@}


struct agl_zone;
struct agl_zone_list;

//! defines an position

/*! This structure defines a pixel position
 */

typedef struct
{
	short x;		//!< x position
	short y;		//!< y position
} agl_pos;

//! defines a vector or a size

/*! This structure defines a vector or a size
 */

typedef struct
{
	unsigned short width;	//!< width size
	unsigned short height;	//!< height size
} agl_size;


//! defines an rectangle

/*! This structure defines a pixel rectangle
 */

typedef struct
{
	short x;		//!< x position
	short y;		//!< y position
	unsigned short width;	//!< rectangle width
	unsigned short height;	//!< rectangle height
} agl_rect;

typedef struct agl_rectlist
{
	agl_rect rect;
	struct agl_rectlist *pnext;
} agl_rectlist;

//! defines an a channel pixmap

/*! This structure defines a map of pixels for one channel of a pixmap
 */

typedef struct
{
	agl_size size;		//!< this rectangles defines the size of the channel
	unsigned char *pixels;	//!< contains all the pixels (a pixel is a 8 bit value)
	unsigned short row_size;	//!< this is the size of a row (pixels) in the pixel table
} agl_channel;

//! defines an a pixmap

/*! This structure defines a map of pixels
 */

typedef struct
{
	agl_size size;		//!< this rectangle defines the size of the pixmap
	unsigned long *pixels;	//!< contains all the pixels (a pixel is a 32 bits value)
	unsigned short row_size;	//!< the size of a row (pixels) in the pixel table
	unsigned char alpha;	//!< defines the tranparency of the pixmap
} agl_pix;


typedef struct agl_zone
{
	agl_pix pix;
	char visible;
	long zl_nb;
	struct agl_zone_list *lower;
	struct agl_zone_list *upper;
	struct agl_zone_list *uplast;
	long flags;
} agl_zone;

typedef struct agl_zone_list
{
	unsigned long nb;
	agl_zone *zone;
	struct agl_zone_list *next;
} agl_zone_list;

typedef struct
{
	agl_pix *pix;
	unsigned long color_key;
	char *image_name;
	char *alpha_name;
} agl_image;

typedef struct screen
{
	agl_zone zone;
	void *sublayer;
	char bpp;
} agl_screen;

typedef struct
{
	int xo;
	int yo;
	int wm;
	int wx;
} character;

//! \name zone flags known
//@{

#define ZONE_TILED	0	//!< defines that the zone sub elements are tiled. they are no overlap calculation
#define ZONE_OVERLAPED	1	//!< defines that the zone sub elements could be overlapped. the sub zones are classified from bottom to top
#define ZONE_CONTAINER	2	//!< defines that the zone contain an image of the sub elements. this could accelerate overlaped zone drawing

//@}

/* image manipulation */

/* Load an image from a data source */

extern agl_image *agl_imgioload(agl_ios * filename);
extern void agl_imgfree(agl_image * image);
extern agl_image *agl_imgnew(int width, int height);
extern agl_image *agl_imgdup(agl_image * pimg);

/* pixmap manipulation */

extern agl_pix *agl_pixnew(int width, int height);
extern void agl_pixfree(agl_pix * ppix);
extern agl_pix *agl_pixdup(agl_pix * ppix);

/* channel manipulation */

extern agl_channel *agl_channew(int width, int height);
extern void agl_chanfree(agl_channel * channel);
extern agl_channel *agl_chandup(agl_channel * channel);
extern int agl_frompixmap(int type, agl_rect * rect, agl_pix * pix, agl_channel ** alpha, agl_channel ** ch1, agl_channel ** ch2,
			  agl_channel ** ch3);
extern int agl_topixmap(int type, agl_rect * rect, agl_pix * pix, agl_channel * alpha, agl_channel * ch1, agl_channel * ch2,
			agl_channel * ch3);


/* zone manipulation */


extern int agl_fillzone(agl_zone * pzone, int width, int height, int flags);
extern void agl_emptyzone(agl_zone * pzone);
extern agl_zone *agl_createzone(int width, int height, int flags);
extern void agl_freezone(agl_zone * zone, int cascade);
extern void agl_originzone(agl_zone * zone, int x, int y);


//! \name transformation types for pixtranscat and pixtransalpha
//@{

#define PIX_ID		0	//!< no transformation
#define PIX_SYM0	1	//!< operate a vertical mirror
#define PIX_SYM45	2	//!< operate a 45 degree diagonal mirror
#define PIX_SYM90	3	//!< operate an horizontal mirror
#define PIX_SYM135	4	//!< operate a 135 degree diagonal mirror
#define PIX_ROT90	5	//!< operate a 90 degree rotation
#define PIX_ROT180	6	//!< operate a 180 degree rotation
#define PIX_ROT270	7	//!< operate a -90 degree rotation

//@}

//! copy one zone into another

/*! copy one zone into another one with clipping of destination zone 
 *  	and source zone alpha of the destination zone is supposed to be opaque
 *	if psrcrect and pdstrect are NULL, no clipping is done
 *	alpha of original dest zone is 255 (opaque)
 * \param psrc the source zone to copy
 * \param pdst the destination zone
 * \param psrcrect the clipping rectangle for the source zone. This
 *  rectangle defines a clipping into the source zone. The coordonates
 *  refers into the source zone
 *  \param pdstrect the clipping rectangle for the destination zone.
 *  \param pos the position of the source zone into the destination zone.
 * \return 
 * - -1 if an error occurs
 * - 0 if nothing bad appears
 */

#define	agl_pixcat(psrc, pdst, psrcrect, pdstrect, pos)	agl_pixtranscat(psrc, pdst, psrcrect, pdstrect, pos, PIX_ID)

//! copy one zone into another with alpha calculation

/*! copy one zone into another one with clipping of destination zone 
   	and source zone and apply alpha to zones
	if psrcrect and pdstrect are NULL, this is a real alpha copy
 * \param psrc the source zone to copy
 * \param pdst the destination zone
 * \param psrcrect the clipping rectangle for the source zone. This
 *  rectangle defines a clipping into the source zone. The coordonates
 *  refers into the source zone
 *  \param pdstrect the clipping rectangle for the destination zone.
 *  \param pos the position of the source zone into the destination zone.
 * \return 
 * - -1 if an error occurs
 * - 0 if nothing bad appears
 */

#define	agl_pixalpha(psrc, pdst, psrcrect, pdstrect, pos)	agl_pixtransalpha(psrc, pdst, psrcrect, pdstrect, pos, PIX_ID)

//! copy one zone into another

/*! copy one zone into another one with clipping of destination zone 
 *  	and source zone alpha of the destination zone is supposed to be opaque
 *	if psrcrect and pdstrect are NULL, no clipping is done
 *	alpha of original dest zone is 255 (opaque)
 *	apply transformation to source before copying to dest 
 * \param psrc the source zone to copy
 * \param pdst the destination zone
 * \param psrcrect the clipping rectangle for the source zone. This
 *  rectangle defines a clipping into the source zone. The coordonates
 *  refers into the source zone
 *  \param pdstrect the clipping rectangle for the destination zone.
 *  \param pos the position of the source zone into the destination zone.
 *  \param type the type of copy.
 * \return 
 * - -1 if an error occurs
 * - 0 if nothing bad appears
 */

extern int agl_pixtranscat(agl_pix * psrc, agl_pix * pdst, agl_rect * psrcrect, agl_rect * pdstrect, agl_pos * pos, int type);

//! copy one zone into another with alpha calculation

/*! copy one zone into another one with clipping of destination zone 
   	and source zone and apply alpha to zones
	if psrcrect and pdstrect are NULL, this is a real alpha copy
 	apply transformation to source before copying to dest
 * \param psrc the source zone to copy
 * \param pdst the destination zone
 * \param psrcrect the clipping rectangle for the source zone. This
 *  rectangle defines a clipping into the source zone. The coordonates
 *  refers into the source zone
 *  \param pdstrect the clipping rectangle for the destination zone.
 *  \param pos the position of the source zone into the destination zone.
 *  \param type the type of copy.
 * \return 
 * - -1 if an error occurs
 * - 0 if nothing bad appears
 */

extern int agl_pixtransalpha(agl_pix * psrc, agl_pix * pdst, agl_rect * psrcrect, agl_rect * pdstrect, agl_pos * pos, int type);

extern int agl_pixfill(agl_pix * ptr, agl_rect * rect, unsigned long color);
extern int agl_pixset(agl_pix * ptr, agl_rect * rect, unsigned long mask, unsigned long color);
extern int agl_pixkey(agl_pix * ptr, agl_rect * rect, unsigned long color);
extern int agl_sublayersetvid(agl_handle handle, int width, int height, int flag);
extern int agl_sublayergetvidsz(agl_handle handle, int flags, int **h, int **w);

#endif /* _agl_video_h */
