# Configure paths
# Stephane APIOU 2004
# stolen from Sam Lantinga

dnl AM_PATH_PACKAGE([MINIMUM-VERSION, [ACTION-IF-FOUND [, ACTION-IF-NOT-FOUND]]])
dnl
AC_DEFUN([AM_PATH_ADGALI],
[dnl 
dnl Get the cflags and libraries from the PACKAGE-config script
dnl
AC_ARG_WITH(adgali-prefix,[  --with-adgali-prefix=PFX   Prefix where ADGALI is installed (optional)],
            adgali_prefix="$withval", adgali_prefix="")
AC_ARG_WITH(adgali-exec-prefix,[  --with-adgali-exec-prefix=PFX Exec prefix where ADGALI is installed (optional)],
            adgali_exec_prefix="$withval", adgali_exec_prefix="")
AC_ARG_ENABLE(adgalitest, [  --disable-adgalitest       Do not try to compile and run a test ADGALI program],
		    , enable_adgalitest=yes)

  if test x$adgali_exec_prefix != x ; then
     adgali_args="$adgali_args --exec-prefix=$adgali_exec_prefix"
     if test x${ADGALI_CONFIG+set} != xset ; then
        ADGALI_CONFIG=$adgali_exec_prefix/bin/adgali-config
     fi
  fi
  if test x$adgali_prefix != x ; then
     adgali_args="$adgali_args --prefix=$adgali_prefix"
     if test x${ADGALI_CONFIG+set} != xset ; then
        ADGALI_CONFIG=$adgali_prefix/bin/adgali-config
     fi
  fi

  AC_PATH_PROG(ADGALI_CONFIG, adgali-config, no)
  min_adgali_version=ifelse([$1], ,0.2.4,$1)
  AC_MSG_CHECKING(for ADGALI - version >= $min_adgali_version)
  no_adgali=""
  if test "$ADGALI_CONFIG" = "no" ; then
    no_adgali=yes
  else
    ADGALI_CFLAGS=`$ADGALI_CONFIG $adgaliconf_args --cflags`
    ADGALI_LIBS=`$ADGALI_CONFIG $adgaliconf_args --libs`

    adgali_major_version=`$ADGALI_CONFIG $adgali_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\1/'`
    adgali_minor_version=`$ADGALI_CONFIG $adgali_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\2/'`
    adgali_micro_version=`$ADGALI_CONFIG $adgali_config_args --version | \
           sed 's/\([[0-9]]*\).\([[0-9]]*\).\([[0-9]]*\)/\3/'`
    if test "x$enable_adgalitest" = "xyes" ; then
      ac_save_CFLAGS="$CFLAGS"
      ac_save_LIBS="$LIBS"
      CFLAGS="$CFLAGS $ADGALI_CFLAGS"
      LIBS="$LIBS $ADGALI_LIBS"
dnl
dnl Now check if the installed ADGALI is sufficiently new. (Also sanity
dnl checks the results of package-config to some extent
dnl
      rm -f conf.adgalitest
      AC_TRY_RUN([
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "agl.h"

char*
my_strdup (char *str)
{
  char *new_str;
  
  if (str)
    {
      new_str = (char *)malloc ((strlen (str) + 1) * sizeof(char));
      strcpy (new_str, str);
    }
  else
    new_str = NULL;
  
  return new_str;
}

int main (int argc, char *argv[])
{
  int major, minor, micro;
  char *tmp_version;

  /* This hangs on some systems (?)
  system ("touch conf.adgalitest");
  */
  { FILE *fp = fopen("conf.adgalitest", "a"); if ( fp ) fclose(fp); }

  /* HP/UX 9 (%@#!) writes to sscanf strings */
  tmp_version = my_strdup("$min_adgali_version");
  if (sscanf(tmp_version, "%d.%d.%d", &major, &minor, &micro) != 3) {
     printf("%s, bad version string
", "$min_adgali_version");
     exit(1);
   }

   if (($adgali_major_version > major) ||
      (($adgali_major_version == major) && ($adgali_minor_version > minor)) ||
      (($adgali_major_version == major) && ($adgali_minor_version == minor) && ($adgali_micro_version >= micro)))
    {
      return 0;
    }
  else
    {
      printf("
*** 'adgali-config --version' returned %d.%d.%d, but the minimum version
", $adgali_major_version, $adgali_minor_version, $adgali_micro_version);
      printf("*** of ADGALI required is %d.%d.%d. If adgali-config is correct, then it is
", major, minor, micro);
      printf("*** best to upgrade to the required version.
");
      printf("*** If adgali-config was wrong, set the environment variable ADGALI_CONFIG
");
      printf("*** to point to the correct copy of adgali-config, and remove the file
");
      printf("*** config.cache before re-running configure
");
      return 1;
    }
}

],, no_adgali=yes,[echo $ac_n "cross compiling; assumed OK... $ac_c"])
       CFLAGS="$ac_save_CFLAGS"
       LIBS="$ac_save_LIBS"
     fi
  fi
  if test "x$no_adgali" = x ; then
     AC_MSG_RESULT(yes)
     ifelse([$2], , :, [$2])     
  else
     AC_MSG_RESULT(no)
     if test "$ADGALI_CONFIG" = "no" ; then
       echo "*** The adgali-config script installed by ADGALI could not be found"
       echo "*** If ADGALI was installed in PREFIX, make sure PREFIX/bin is in"
       echo "*** your path, or set the ADGALI_CONFIG environment variable to the"
       echo "*** full path to adgali-config."
     else
       if test -f conf.adgalitest ; then
        :
       else
          echo "*** Could not run ADGALI test program, checking why..."
          CFLAGS="$CFLAGS $ADGALI_CFLAGS"
          LIBS="$LIBS $ADGALI_LIBS"
          AC_TRY_LINK([
#include <stdio.h>
#include "agl.h"
],      [ return 0; ],
        [ echo "*** The test program compiled, but did not run. This usually means"
          echo "*** that the run-time linker is not finding ADGALI or finding the wrong"
          echo "*** version of ADGALI. If it is not finding ADGALI, you'll need to set your"
          echo "*** LD_LIBRARY_PATH environment variable, or edit /etc/ld.so.conf to point"
          echo "*** to the installed location  Also, make sure you have run ldconfig if that"
          echo "*** is required on your system"
	  echo "***"
          echo "*** If you have an old version installed, it is best to remove it, although"
          echo "*** you may also be able to get things to work by modifying LD_LIBRARY_PATH"],
        [ echo "*** The test program failed to compile or link. See the file config.log for the"
          echo "*** exact error that occured. This usually means ADGALI was incorrectly installed"
          echo "*** or that you have moved ADGALI since it was installed. In the latter case, you"
          echo "*** may want to edit the adgali-config script: $ADGALI_CONFIG" ])
          CFLAGS="$ac_save_CFLAGS"
          LIBS="$ac_save_LIBS"
       fi
     fi
     ADGALI_CFLAGS=""
     ADGALI_LIBS=""
     ifelse([$3], , :, [$3])
  fi
  AC_SUBST(ADGALI_CFLAGS)
  AC_SUBST(ADGALI_LIBS)
  rm -f conf.adgalitest
])
